<?
/***Copyright and coded by Dakusan - See http://www.castledragmire.com/Copyright for more information. ***/
/***Dakusan's MySQL Library - v1.0 http://www.castledragmire.com/Projects/Other_Web_Scripts#MySQL ***/

//Exits out of php, printing an error
if(!function_exists('SQLError'))
{
	function SQLError($Str, $Query) { print "SQL Error encountered, please contact the administrators with the following information: $Str\n\nTechnical Data:\n$Query"; exit; }
}

//MySQL functions
function MysqlOpen($Server=NULL, $UserName=NULL, $Password=NULL, $Database=NULL)
{
	//Use global SQLInfo if information is not passed in
	global $SQLInfo, $link;
	$CredentialNames=Array('Server', 'UserName', 'Password', 'Database');
	$SQLInfo=(isset($SQLInfo) ? $SQLInfo : Array());
	foreach($CredentialNames as $Name)
		$SQLInfo[$Name]=(!is_null(${$Name}) ? ${$Name} : (isset($SQLInfo[$Name]) ? $SQLInfo[$Name] : NULL));
	foreach(Array('QueryDebug'=>FALSE, 'QueryRememberData'=>FALSE) as $Name => $Value) //Confirm other required members
		$SQLInfo[$Name]=(isset($SQLInfo[$Name]) ? $SQLInfo[$Name] : $Value);
	$SQLInfo['Server']=(!is_null($SQLInfo['Server']) ? $SQLInfo['Server'] : 'localhost'); //Set server to localhost by default

	//Only open SQL if not already opened
	if(!isset($SQLInfo['Count']) || $SQLInfo['Count']==0)
	{
		if(!($SQLInfo['Reference']=$link=mysqli_connect($SQLInfo['Server'], $SQLInfo['UserName'], $SQLInfo['Password'], $SQLInfo['Database'])))
			die('Connect failed: '.mysqli_connect_error());
		$SQLInfo['Count']=0;
		
		//Make sure connection is set as UTF8
		mysqli_query($SQLInfo['Reference'], "SET NAMES 'utf8' COLLATE 'utf8_general_ci'");
		mysqli_query($SQLInfo['Reference'], "SET CHARACTER SET 'utf8'");
	}
	$SQLInfo['Count']++; //Keep track of number of opened instances
}

function MysqlClose($ForceClose=false)
{
	global $SQLInfo;
	$SQLInfo['Count']=($ForceClose?0:$SQLInfo['Count']-1); //Either decrement the counter of set to 0 if close is forced
	if($SQLInfo['Count']) //If counter is not at 0, do nothing else
		return;
	//Close SQL
	mysqli_close($SQLInfo['Reference']);
	unset($SQLInfo['Reference']);
}

function GetDataTypes(&$Data) //Returns a string of MySQL datatypes from array $Data
{
	$DataTypes='';
	foreach($Data as $K => $D)
		if(is_int($D))
			$DataTypes.='i';
		else if(is_double($D))
			$DataTypes.='d';
		else
			$DataTypes.='s';
	return $DataTypes;
}

function MysqlQuery($TheQuery)
{
	//Confirms connection
	global $SQLInfo;
	if(!isset($SQLInfo) || !isset($SQLInfo['Reference'])) //If SQL is not opened, open it
		MysqlOpen();

	//Allows a user override function to modify the query
	if(function_exists('MysqlQueryOverride'))
		$TheQuery=MysqlQueryOverride($TheQuery);

	//Total count of number of queries
	if($SQLInfo['QueryDebug'])
	{
		static $TotalTime=0;
		static $QueryTimes=Array();
		if($TheQuery=='GetQueryCount')
			return Array(count($QueryTimes),$TotalTime);
		else if($TheQuery=='GetQueryTimes')
			return $QueryTimes;
		$StartTime=microtime();
	}

	$Data=array_flatten(array_slice(func_get_args(), 1)); //Get a flattened array of all data past the first query item
	if($SQLInfo['QueryRememberData']) //Remember the original data if its needed below
		$OriginalData=$Data;
	if(!count($Data)) //If only 1 argument, run a normal query
	{
		$stmt=mysqli_query($SQLInfo['Reference'], $TheQuery);
		if($stmt===FALSE)
			SQLError(mysqli_error($SQLInfo['Reference']), $TheQuery);
	}
	else //Otherwise run a prepared query
	{
		$stmt=mysqli_prepare($SQLInfo['Reference'], $TheQuery); //Prepare the statement	
		$DataTypes=GetDataTypes($Data); //Get the data types
		array_unshift($Data, $stmt, $DataTypes); //Prepare a full array of all variables to pass to bind parameter function
		call_user_func_array('mysqli_stmt_bind_param', RefValues($Data)); //Run the bind function
		if(!mysqli_stmt_execute($stmt) || !mysqli_stmt_store_result($stmt)) //Execute the statement
			SQLError(mysqli_error($SQLInfo['Reference']), $TheQuery);
	}
		
	//Remember query information for diagnostics
	if($SQLInfo['QueryDebug'])
	{
		$TotalTime+=($MyTime=MicroTimeDiff($StartTime, microtime()));
		$QueryTimes[]=Array($MyTime,$TheQuery, $SQLInfo['QueryRememberData'] ? $OriginalData : FALSE);
	}
	
	return $stmt; //Return the statement
}

function MysqlFetchData($R, $RowNum='')
{
	$ReturnArray=Array();
	if($R instanceof mysqli_stmt) //Is a statement
	{
		//Prepare to receive fields
		$Fs=mysqli_stmt_result_metadata($R);
		$D=Array();
		$DList=Array($R);
		$CopyArray=Array();
		$i=0;

		//Get the fields and pointers to an array containing the fields for the statement to bind to
		while($field=mysqli_fetch_field($Fs))
			$DList[++$i]=&$D[$field->name];
		$NumFields=count($D);
		call_user_func_array('mysqli_stmt_bind_result', $DList);
		while(mysqli_stmt_fetch($R)===TRUE)
			if($NumFields==1)
				$ReturnArray[]=current($D);
			else
			{
				foreach($D as $K => $V) //We need to duplicate the array to remove the fact that each member is referenced
					$CopyArray[$K]=$V;
				$ReturnArray[]=$CopyArray;
			}
	}
	else //Is a result
	{
		if($R===FALSE) //If no result set, return FALSE
			return FALSE;
		else if(($D=mysqli_fetch_assoc($R))===NULL) //If we don't have any information, return an empty array
			return Array();
		$NumFields=count($D);
		do
			$ReturnArray[]=($NumFields==1 ? current($D) : $D);
		while(($D=mysqli_fetch_assoc($R))!==NULL);
		mysqli_free_result($R);
	}
	return $RowNum!=='' ? (array_key_exists($RowNum, $ReturnArray) ? $ReturnArray[$RowNum] : FALSE) : $ReturnArray;
}

//Query preparation functions
function MysqlPrepareList($List) { return implode(', ', array_fill(0, count($List), '?')); } //Returns a string of question marks separated by commas whos list length is equal to the array length of the parameter
function MysqlPrepareUpdateList($NameList) { return implode('=?, ', $NameList).'=?'; } //Returns a string in the format "NAME=?, NAME=?, ..., NAME=?"

//Escape functions
function EHTML($S) { return htmlspecialchars($S, ENT_QUOTES, 'UTF-8'); }
function EJS($TheString, $IsClosed=TRUE)
{
	$IsClosed=($IsClosed ? '"' : '');
	return $IsClosed.strtr($TheString, Array('\\'=>'\\\\', '/'=>'\/', '"'=>'\"', "'"=>"\\'", "\r"=>'\r', "\n"=>'\n')).$IsClosed;
}

//Array functions
function array_flatten($Arr)
{
	$ReturnArray=Array();
	foreach($Arr as $Val)
		if(is_array($Val))
			$ReturnArray=array_merge($ReturnArray,array_flatten($Val));
		else
			$ReturnArray[]=$Val;
	return $ReturnArray;
}

//Variable functions
function RefValues($Data) //Turns values in an array to references in place
{
	if(strnatcmp(phpversion(),'5.3')>=0) //Reference required for PHP 5.3+
		foreach($Data as $Key => $Value)
			$Data[$Key] = &$Data[$Key];
	return $Data;
}

//Time Functions
function MicroTimeDiff($StartTime, $EndTime) //Returns the second based decimal difference of two microtime() statements
{
	$StartTime=explode(' ', $StartTime);
	$EndTime=explode(' ', $EndTime);
	return ($EndTime[0]-$StartTime[0])+($EndTime[1]-$StartTime[1]);
}
?>